package com.flyfish.framework.validation.validators;


import com.flyfish.framework.validation.annotations.UniqueField;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.ClassUtils;
import org.springframework.beans.BeanUtils;

import javax.validation.ConstraintValidator;
import javax.validation.ConstraintValidatorContext;
import java.beans.PropertyDescriptor;
import java.lang.reflect.InvocationTargetException;
import java.util.Collection;
import java.util.Map;

public class UniqueFieldValidator implements ConstraintValidator<UniqueField, Collection> {

    private String field;

    @Override
    public void initialize(UniqueField uniqueField) {
        this.field = uniqueField.value();
    }

    /**
     * Implements the validation logic.
     * The state of {@code value} must not be altered.
     * <p>
     * This method can be accessed concurrently, thread-safety must be ensured
     * by the implementation.
     *
     * @param value   object to validate
     * @param context context in which the constraint is evaluated
     * @return {@code false} if {@code value} does not pass the constraint
     */
    @Override
    public boolean isValid(Collection value, ConstraintValidatorContext context) {
        int size = CollectionUtils.size(value);
        if (size < 2) {
            return true;
        }
        long count = ((Collection<?>) value).stream().map(item -> {
            if (item instanceof String || ClassUtils.isPrimitiveOrWrapper(item.getClass())) {
                return item;
            }
            if (item instanceof Map) {
                return ((Map<?, ?>) item).get(field);
            }
            try {
                PropertyDescriptor descriptor = BeanUtils.getPropertyDescriptor(item.getClass(), field);
                if (null != descriptor) {
                    return descriptor.getReadMethod().invoke(item);
                }
                return null;
            } catch (IllegalAccessException | InvocationTargetException ignored) {
                return null;
            }
        }).distinct().count();
        return count == size;
    }
}
