package com.flyfish.framework.logging.service;

import com.alibaba.fastjson.JSON;
import com.flyfish.framework.logging.config.LoggingTextRegistry;
import com.flyfish.framework.logging.domain.Log;
import com.flyfish.framework.logging.domain.LogType;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.ArrayUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.DisposableBean;

import javax.annotation.Resource;
import java.util.Date;
import java.util.Optional;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

/**
 * 日志管理器
 *
 * @author wangyu
 */
@Slf4j
public class LogManager implements DisposableBean {

    // 固定线程池
    private final ExecutorService executorService = Executors.newFixedThreadPool(20);
    @Resource
    private LogService logService;
    @Resource
    private LoggingTextRegistry registry;

    /**
     * 尝试记录日志
     *
     * @param context 上下文
     */
    public void tryLog(LogContext context) {
        executorService.execute(() -> {
            // 构建日志
            Log log = new Log();
            log.setType(LogType.OPERATION);
            log.setSuccess(context.isSuccess());
            log.setBody(bodyString(context.getArgs()));
            log.setModule(context.getModule());
            log.setBusiness(Optional.ofNullable(context.getBusiness())
                    .filter(StringUtils::isNotBlank)
                    .map(registry::text)
                    .orElse(context.getBusiness()));
            log.setError(Optional.ofNullable(context.getError()).map(JSON::toJSONString).orElse(null));
            log.setResponse(Optional.ofNullable(context.getResult()).map(JSON::toJSONString).orElse(null));
            log.setOperator(context.getUser());
            log.setSignature(context.getSignature());
            log.setPeriod(context.getEndTime() - context.getStartTime());
            log.setStartTime(new Date(context.getStartTime()));
            log.setEndTime(new Date(context.getEndTime()));
            // 先不拼接
            log.setUri(null);
            // 写入日志
            logService.create(log).subscribe();
        });
    }

    private String bodyString(Object[] args) {
        if (ArrayUtils.isEmpty(args)) {
            return null;
        }
        if (args.length == 1) {
            return JSON.toJSONString(args[0]);
        }
        return JSON.toJSONString(args);
    }

    @Override
    public void destroy() {
        if (!executorService.isShutdown()) {
            executorService.shutdownNow();
        }
    }
}
