package com.flyfish.framework.beans.meta.parser;

import com.flyfish.framework.beans.meta.parser.chain.BeanPropertyAnnotationBatchChain;
import com.flyfish.framework.beans.meta.parser.chain.BeanPropertyAnnotationChain;
import com.flyfish.framework.beans.meta.parser.chain.impl.EmptyAnnotationBatchChain;
import com.flyfish.framework.beans.meta.parser.chain.impl.EmptyAnnotationChain;
import com.flyfish.framework.beans.meta.parser.chain.impl.SimpleAnnotationBatchChain;
import com.flyfish.framework.beans.meta.parser.chain.impl.SimpleAnnotationChain;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.core.annotation.MergedAnnotation;
import org.springframework.core.annotation.MergedAnnotations;
import org.springframework.data.util.CastUtils;

import java.lang.annotation.Annotation;
import java.lang.reflect.Field;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

/**
 * bean属性注解容器
 *
 * @author wangyu
 */
public class SimpleBeanPropertyAnnotations implements BeanPropertyAnnotations {

    private final Field field;

    private final MergedAnnotations annotations;
    private final EmptyAnnotationChain<? extends Annotation, ?> emptyChain = new EmptyAnnotationChain<>(this);
    private final EmptyAnnotationBatchChain<? extends Annotation, ?> emptyBatchChain = new EmptyAnnotationBatchChain<>(this);
    private boolean last;

    public SimpleBeanPropertyAnnotations(Field field) {
        this.field = field;
        this.annotations = null == field ? MergedAnnotations.from() : MergedAnnotations.from(field);
    }

    /**
     * 判断是否有某个注解
     *
     * @param annotationType 注解类型
     * @return 结果
     */
    @Override
    public <A extends Annotation> boolean isPresent(Class<A> annotationType) {
        if (null == field) {
            return false;
        }
        return annotations.isPresent(annotationType);
    }

    /**
     * 获取纯粹的spring注解属性表
     *
     * @return 结果
     */
    @Override
    public MergedAnnotations annotations() {
        return annotations;
    }

    /**
     * 判断某个注解是否存在，存在则交给consumer进行处理，仅取得找到的第一个注解
     *
     * @param annotationType 注解类型
     * @return 结果
     */
    @Override
    public <A extends Annotation> BeanPropertyAnnotationChain<A, MergedAnnotation<A>> as(Class<A> annotationType) {
        if (null != field && !this.last) {
            MergedAnnotation<A> annotation = this.annotations.get(annotationType);
            if (annotation.isPresent()) {
                return new SimpleAnnotationChain<>(this.last(true), annotation);
            }
        }
        return empty();
    }

    /**
     * 判断某个注解是否存在，存在则交给consumer进行处理，取得所有注解并自动判断复数形式
     *
     * @param annotationType 注解类型
     * @param listType       列表类型
     * @param <A>            泛型
     * @param <L>            列表泛型
     * @return 结果
     */
    @Override
    public <A extends Annotation, L extends Annotation> BeanPropertyAnnotationBatchChain<A, MergedAnnotation<A>> as(
            Class<A> annotationType, Class<L> listType) {
        if (null != field && !this.last) {
            MergedAnnotation<L> annotation = this.annotations.get(listType);
            if (annotation.isPresent()) {
                return new SimpleAnnotationBatchChain<>(this.last(true),
                        Arrays.asList(annotation.getAnnotationArray("value", annotationType)));
            } else {
                List<MergedAnnotation<A>> values = this.annotations.stream(annotationType)
                        .filter(MergedAnnotation::isPresent)
                        .collect(Collectors.toList());
                if (CollectionUtils.isNotEmpty(values)) {
                    return new SimpleAnnotationBatchChain<>(this.last(true), values);
                }
            }
        }
        return batchEmpty();
    }

    /**
     * 设置上一步结果
     *
     * @param success 是否成功
     * @return 结果
     */
    @Override
    public SimpleBeanPropertyAnnotations last(boolean success) {
        this.last = success;
        return this;
    }

    /**
     * 返回上一步成功结果
     *
     * @return 结果
     */
    @Override
    public boolean last() {
        return this.last;
    }

    /**
     * 返回空的运行链
     *
     * @return 结果
     */
    @Override
    public <A extends Annotation, E> BeanPropertyAnnotationChain<A, E> empty() {
        return CastUtils.cast(emptyChain);
    }

    /**
     * 返回空的运行链
     *
     * @return 结果
     */
    @Override
    public <A extends Annotation, E> BeanPropertyAnnotationBatchChain<A, E> batchEmpty() {
        return CastUtils.cast(emptyBatchChain);
    }
}
