package com.flyfish.framework.beans.repository;

import com.flyfish.framework.repository.DefaultReactiveRepository;
import com.flyfish.framework.repository.DefaultRepository;
import org.apache.commons.lang3.ClassUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.FactoryBean;
import org.springframework.beans.factory.config.BeanDefinition;
import org.springframework.beans.factory.support.AbstractBeanDefinition;
import org.springframework.beans.factory.support.BeanDefinitionBuilder;
import org.springframework.beans.factory.support.BeanDefinitionRegistry;
import org.springframework.beans.factory.support.BeanNameGenerator;
import org.springframework.core.env.Environment;
import org.springframework.core.io.ResourceLoader;
import org.springframework.core.type.AnnotationMetadata;
import org.springframework.data.mongodb.repository.config.EnableMongoRepositories;
import org.springframework.data.mongodb.repository.config.EnableReactiveMongoRepositories;
import org.springframework.data.mongodb.repository.config.MongoRepositoryConfigurationExtension;
import org.springframework.data.mongodb.repository.config.ReactiveMongoRepositoryConfigurationExtension;
import org.springframework.data.repository.config.*;

/**
 * 自定义的仓库注册器
 *
 * @author wangyu
 */
public class CustomRepositoryRegistrar {

    // repo生成的extension
    private final RepositoryConfigurationExtension extension = new MongoRepositoryConfigurationExtension();
    private final RepositoryConfigurationSource configurationSource;
    private final CustomRepositoryBuilder builder;
    // 异步的extension
    private final RepositoryConfigurationExtension reactiveExtension = new ReactiveMongoRepositoryConfigurationExtension();
    private final RepositoryConfigurationSource reactiveConfigurationSource;
    private final CustomRepositoryBuilder reactiveBuilder;

    public CustomRepositoryRegistrar(AnnotationMetadata metadata, ResourceLoader resourceLoader, Environment environment,
                                     BeanDefinitionRegistry registry, BeanNameGenerator generator) {
        // repo生成的构建器
        this.configurationSource = new AnnotationRepositoryConfigurationSource(metadata,
                EnableMongoRepositories.class, resourceLoader, environment, registry, generator);
        // 异步的repo生成的构建器
        this.reactiveConfigurationSource = new AnnotationRepositoryConfigurationSource(metadata,
                EnableReactiveMongoRepositories.class, resourceLoader, environment, registry, generator);
        // 构建器
        this.builder = new CustomRepositoryBuilder(registry, extension,
                configurationSource, resourceLoader, environment);
        // 异步的构造器
        this.reactiveBuilder = new CustomRepositoryBuilder(registry, reactiveExtension, reactiveConfigurationSource,
                resourceLoader, environment);
    }

    /**
     * 根据class直接注册bean
     *
     * @param clazz 编译后的class
     */
    public void register(Class<?> clazz) {
        if (ClassUtils.isAssignable(clazz, DefaultRepository.class)) {
            register(clazz, configurationSource, extension, builder);
        } else if (ClassUtils.isAssignable(clazz, DefaultReactiveRepository.class)) {
            register(clazz, reactiveConfigurationSource, reactiveExtension, reactiveBuilder);
        }
    }

    /**
     * 注册仓库内部方法，区分异步和同步
     *
     * @param clazz               类型
     * @param configurationSource 配置源
     * @param extension           扩展
     * @param builder             构建器
     */
    private void register(Class<?> clazz, RepositoryConfigurationSource configurationSource, RepositoryConfigurationExtension extension, CustomRepositoryBuilder builder) {
        //生成BeanDefinition并注册到容器中
        BeanDefinition origin = BeanDefinitionBuilder.genericBeanDefinition(clazz).getRawBeanDefinition();
        //设置当前bean定义对象是单例的
        origin.setScope("singleton");
        RepositoryConfiguration<? extends RepositoryConfigurationSource> configuration =
                new DefaultRepositoryConfiguration<>(configurationSource, origin, extension);
        BeanDefinitionBuilder definitionBuilder = builder.build(configuration);
        extension.postProcess(definitionBuilder, configurationSource);
        extension.postProcess(definitionBuilder, (AnnotationRepositoryConfigurationSource) configurationSource);

        AbstractBeanDefinition beanDefinition = definitionBuilder.getBeanDefinition();
        beanDefinition.setPrimary(configuration.isPrimary());
        String beanName = StringUtils.uncapitalize(clazz.getSimpleName());
        beanDefinition.setAttribute(FactoryBean.OBJECT_TYPE_ATTRIBUTE, configuration.getRepositoryInterface());
        builder.getRegistry().registerBeanDefinition(beanName, beanDefinition);
    }


    public RepositoryConfigurationExtension getExtension() {
        return extension;
    }
}
